@@ -29,7 +29,7 @@ get_incidence_from_adjacency <- function(A) {
 #' @param record_objective whether or not to record the objective function. Default is FALSE
 #' @param verbose if TRUE, then a progress bar will be displayed in the console. Default is TRUE
 #' @return A list containing possibly the following elements:
-#' \item{\code{Laplacian}}{the estimated Laplacian Matrix}
+#' \item{\code{laplacian}}{the estimated Laplacian Matrix}
 #' \item{\code{Adjacency}}{the estimated Adjacency Matrix}
 #' \item{\code{convergence}}{boolean flag to indicate whether or not the optimization converged}
 #' \item{\code{obj_fun}}{values of the objective function at every iteration in case record_objective = TRUE}
@@ -53,7 +53,7 @@ learn_laplacian_gle_mm <- function(S, A_mask = NULL, alpha = 0, maxiter = 10000,
   m <- sum(mask)#.5 * sum(A_mask > 0)
   # l1-norm penalty factor
   J <- matrix(1, p, p) / p
-  H <- 2 * diag(p) - p * J
+  H <- diag(p) - p * J
   K <- S + alpha * H
   E <- get_incidence_from_adjacency(A_mask)
   R <- t(E) %*% K %*% E
@@ -87,7 +87,7 @@ learn_laplacian_gle_mm <- function(S, A_mask = NULL, alpha = 0, maxiter = 10000,
   }
   z <- rep(0, .5 * p * (p - 1))
   z[mask] <- wk
-  results <- list(Laplacian = L(z), Adjacency = A(z), maxiter = k,
+  results <- list(laplacian = L(z), adjacency = A(z), maxiter = k,
                   convergence = has_converged)
   if (record_objective)
     results$obj_fun <- fun
@@ -139,13 +139,15 @@ learn_laplacian_gle_admm <- function(S, A_mask = NULL, alpha = 0, rho = 1, maxit
   tau <- 2
   # l1-norm penalty factor
   J <- matrix(1, p, p) / p
-  H <- 2 * diag(p) - p * J
+  H <- diag(p) - p * J
   K <- S + alpha * H
   if (verbose)
     pb <- progress::progress_bar$new(format = "<:bar> :current/:total  eta: :eta",
                                      total = maxiter, clear = FALSE, width = 80)
   if (record_objective)
     fun <- c(vanilla.objective(Theta, K))
+  time_seq <- c(0)
+  start_time <- proc.time()[3]
   # ADMM loop
   P <- qr.Q(qr(rep(1, p)), complete=TRUE)[, 2:p]
   for (k in c(1:maxiter)) {
@@ -156,13 +158,13 @@ learn_laplacian_gle_admm <- function(S, A_mask = NULL, alpha = 0, rho = 1, maxit
     Xik <- crossprod(sqrt(d) * t(U))
     Thetak <- P %*% Xik %*% t(P)
     Ck_tmp <- Yk / rho + Thetak
-    Ck <- (diag(pmax(0, diag(Ck_tmp))) +
-           A_mask * pmin(0, Ck_tmp))
+    Ck <- diag(pmax(0, diag(Ck_tmp))) + A_mask * pmin(0, Ck_tmp)
     Rk <- Thetak - Ck
     Yk <- Yk + rho * Rk
     if (record_objective)
       fun <- c(fun, vanilla.objective(Thetak, K))
-    has_converged <-  norm(Theta - Thetak) / norm(Theta, "F") < reltol
+    has_converged <-  (norm(Theta - Thetak) / norm(Theta, "F") < reltol) & (k > 1)
+    time_seq <- c(time_seq, proc.time()[3] - start_time)
     if (has_converged && k > 1) break
     s <- rho * norm(C - Ck, "F")
     r <- norm(Rk, "F")
@@ -175,8 +177,8 @@ learn_laplacian_gle_admm <- function(S, A_mask = NULL, alpha = 0, rho = 1, maxit
     if (verbose)
       pb$tick()
   }
-  results <- list(Laplacian = Thetak, Adjacency = diag(diag(Thetak)) - Thetak,
-                  convergence = has_converged)
+  results <- list(laplacian = Thetak, adjacency = diag(diag(Thetak)) - Thetak,
+                  convergence = has_converged, elapsed_time = time_seq)
   if (record_objective)
     results$obj_fun <- fun
   return(results)
@@ -185,6 +187,6 @@ learn_laplacian_gle_admm <- function(S, A_mask = NULL, alpha = 0, rho = 1, maxit
 # compute the partial augmented Lagragian
 aug_lag <- function(K, P, Xi, Y, C, d, rho) {
   PXiPt <- P %*% Xi %*% t(P)
-  return(sum(diag(Xi %*% t(P) %*% K %*% P)) - sum(log(d)) +
-         sum(diag(t(Y) %*% (PXiPt - C))) + .5 * rho * norm(PXiPt - C, "F") ^ 2)
+  return((sum(diag(Xi %*% t(P) %*% K %*% P)) - sum(log(d))
+          + sum(diag(t(Y) %*% (PXiPt - C))) + .5 * rho * norm(PXiPt - C, "F") ^ 2))
 }
